// naumann@stce.rwth-aachen.de

#pragma once

#include "config.h"
#include "objective.h"

/// Stationary point of objective wrt. model parameter by bisection
template<typename T>
T fit(T p, VT<T> xobs, VT<T> yobs) {
  // conditions
  assert(xobs.size()==nobs);
  assert(xobs.size()==yobs.size());
  // search interval (smarter search required in general)
  T p_up=p+10; 
  p=p-10; 
  // iterative bisection to decrease width of search interval
  while (fabs(p-p_up)>eps) {
    // midpoint of search interval
    T p_mid=(p_up+p)/2; 
    // derivative of objective at lower bound
    T dedp=de_dp(p,xobs,yobs);
    // derivative of objective at midpoint
    T dedp_mid=de_dp(p_mid,xobs,yobs);
    // collapse search interval if stationary point found
    if (fabs(dedp_mid)<eps) { 
      p=p_up=p_mid;
    }
    // continue bisection if stationary point not found
    else if ((dedp<=0&&dedp_mid<=0)||(dedp>=0&&dedp_mid>=0)) {
      // search in right half of sign switches there
      p=p_mid; 
    } else {
      // search in left half otherwise
      p_up=p_mid; 
    }
  }
  return p;
}
