// naumann@stce.rwth-aachen.de

#pragma once

#include "config.h"
#include "objective.h"

/// stationary point of objective wrt. model parameter by the gradient descent method
template<typename T>
T fit(T p, VT<T> xobs, VT<T> yobs) {
  // conditions
  assert(xobs.size()==nobs);
  assert(xobs.size()==yobs.size());
  // derivative of objective wrt. model parameter
  T dedp=de_dp(p,xobs,yobs); 
  // search for stationary point of objective wrt. model parameter
  do {
    // simple line search
    T alpha=1;
    T e_=e(p,xobs,yobs); 
    while (e(p-alpha*dedp,xobs,yobs)>=e_) { 
      alpha=alpha/2; 
    }
    // gradient descent step
    p=p-alpha*dedp; 
    dedp=de_dp(p,xobs,yobs); 
  } while (fabs(dedp)>eps); // convergence criterion
  return p;
}
