// naumann@stce.rwth-aachen.de

#pragma once

#include "config.h"
#include "objective.h"

/// stationary point of objective wrt. model parameter by Newton method
template<typename T>
T fit(T p, VT<T> xobs, VT<T> yobs) {
  // conditions
  assert(xobs.size()==nobs);
  assert(xobs.size()==yobs.size());
  // derivative of objective wrt. model parameter
  T dedp=de_dp(p,xobs,yobs);
  do {
    // Newton step requires second derivative of objective wrt. model parameter
    p=p-dedp/dde_dp_dp(p,xobs,yobs);
    dedp=de_dp(p,xobs,yobs);
  } while (fabs(dedp)>eps);
  return p;
}
