// naumann@stce.rwth-aachen.de

#pragma once

#include "config.h"

#include "dco.hpp"

/// model
template<typename T> 
T f(T p, T x);

/// residual of model predictions relative to observations
template<typename T>
VT<T> r(T p, VT<T> xobs, VT<T> yobs) {
  // conditions
  assert(xobs.size()==nobs);
  assert(xobs.size()==yobs.size());
  // return value
  VT<T> r(nobs);
  int i=0;
  while (i<nobs) {
    // model error relative to observations 
    r(i)=f(p,xobs(i))-yobs(i);
    i=i+1;
  }
  return r;
}

/// derivative of residual wrt. model parameter 
template<typename T> 
VT<T> dr_dp(T p, VT<T> xobs) { 
  // conditions
  assert(xobs.size()==nobs);
  // return value
  VT<T> dr_dp(nobs);
  // tangent type
  using TT=dco::gt1s<T>::type;
  int i=0;
  while (i<nobs) {
    // activation
    TT p_t=p, y_t;
    // seeding derivative of model parameter
    dco::derivative(p_t)=1;
    // overloaded computation of model value at current point of observation 
    y_t=f<TT>(p_t,xobs(i));
    // harvesting derivative of residual wrt. model parameter at current point of observation 
    dr_dp(i)=dco::derivative(y_t);
    i=i+1;
  }
  return dr_dp;
}
